/* --------------------------------------------------------------------- */
/* 
   SCCS information: %W% %G%
*/
/* --------------------------------------------------------------------- */

/*  These subroutines invert the pressure equation

  ON INPUT:
 
  nx by ny double arrays:
  ---------------------
  u:  initial right hand side (divergence field)
  p:  anything
  cc: concentrations or volume fractions of phase 1
  a:  coefficients for equation on grid n, computed with "afromc"
  b:  coefficients "   "
  ei: coefficients inverse "  "

  scalar parameters:
  ------------------
  n:      dimension of arrays

 ON OUTPUT:

  everything the same, except

  p:      contains the pressure 
*/

#ifndef MG_H
#define MG_H

#define NGERROR		-1
#define NXERROR		-2
#define NYERROR		-3
#define MEMERROR	-5


extern void relax(real2D u, real2D rhs, 
		  real2D a, real2D b, real2D ei, real2D cc,
		  int n1, int n2);

extern int computeng(int n1, int n2);

extern int initpressure(int n1, int n2, int ng);

extern void mginit(real2D u, real2D p, real2D a, real2D b, real2D ei, 
		   int n1, int n2, int ng); 

extern void mgvcycle(int n, int nx, int ny, int npre, int npost);

extern void mgfas(real2D u, real2D p, real2D a, real2D b, real2D ei, 
		  int n1, int n2, int ng, int npre, int npost, int ncycle);
#endif

