#include "interpolation.h"

#define NOTENSION 0
#define TENSION_U 1
#define TENSION_V 2
#define NSEC 10
#define MASSPREC 1.e-3

#define TRUE_PERIODIC 1
#define APPROX_PERIODIC 2
#define AXITRUE 3
#define AXINATURAL 4
#define AXIBOTH 5

#define INTERPREC 1.e-6
#define MARK_TYPE 0
#define HOR_TYPE 1
#define VER_TYPE 2

typedef struct {
  real *x, *y, *t;
  polynom3 *spx, *spy;
#ifdef FREE
  real *p;
#endif
  int n;
  char bc;
} interface;


typedef struct {
  int n;
  int j1[NSEC], j2[NSEC], in[NSEC];
} intersec;

extern int interface_arc(interface in, real t);

extern void interface_write_markers(interface in, FILE *fptr);

extern void interface_write_markers_speed(real time, interface in, real2D u, 
		real2D v, real tau, real du, 
		int nx, int ny, FILE *fptr);

extern void interface_write_splines(interface in, FILE *fptr, int nb);

extern void curvature_write(interface in, FILE *fptr);

extern real interfaces_area(interface *in, int n);

extern real interfaces_axivolume(interface *in, int n);

extern real interfaces_xmoment(interface *in, int n);

extern real interfaces_ymoment(interface *in, int n);

extern real interfaces_x_amp(interface *in, int n);

extern real interfaces_y_amp(interface *in, int n);

extern void interface_init(interface *dst);

/* Approximately 60000 pts/sec on an Indigo2
   1 100 MHZ IP22 Processor
   FPU: MIPS R4010 Floating Point Chip Revision: 0.0
   CPU: MIPS R4000 Processor Chip Revision: 3.0 */
extern void interface_splines(interface in, int nx, int ny);

extern void interface_redis(interface *in, real space);

extern void interface_redis_scale(interface *in, interface src, real space, 
                                  real scale);

extern void interface_axisymmetry(interface *in, real space, int nx, int ny);

extern int reconnect_axis(interface *in, int nx, int ny, real length);

extern void interface_copy(interface *dst, interface in);

extern void interface_free(interface in);

extern void interface_advect(interface in, real2D u, real2D v, 
#ifdef FREE
                             real2D a, real2D c, real2D cc,
#endif
                             int nx, int ny);

extern void interface_to_c(interface *in, int nint, 
#ifdef RECON
			   intersec **inter, 
#endif
			   real2D work, real2D c, real xo, real yo,
			   int nx, int ny, int sflag,
			   real2D rhox, real2D p, real sigma, real tau);

extern real interface_inside(interface in, real x, real y);

extern void interface_fill(interface *in, int nin, real2D ap, int nx, int ny);

extern int interface_split_axis(interface **inlist, real length);

extern int interface_merge_axis(interface **inlist, real length);

extern real bilinu(real2D u, real x1, real y1);

extern real bilinv(real2D v, real x1, real y1);
