/*----------------------------------------------------------------------------*/
/* SCCS Information: %W%    %G% */
/*----------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "utilf.h"
#include "markers.h"
#include "xfig.h"
#include "printfree.h"
#include "inout.h"


#define SCALE 100.0
#define XFIG(x) (SCALE*(x) + 100.)
#define YFIG(y) (SCALE*(y) + 100.)

void printfree(FILE *fptr,
	      real2D u, real2D v, real2D ap,
	      interface in,
	      int nx, int ny)
{
  int i, j, *x, *y;
  real maxu, maxv;
  char s[256];
  int xs = 2, xe = nx - 1, ys = 2, ye = ny - 1;

  fprintf(fptr, "#FIG 2.1\n80 1\n");
  for (i = xs; i <= xe; i++) {
    XfigLine(fptr, XFIG((real)i), YFIG(ys), XFIG((real)i), YFIG(ye),
	     XFIG_SOLID, 1, XFIG_BLACK);
    sprintf(s, "%d", i);
    XfigText(fptr, XFIG((real)i + 0.5), YFIG((real)ys - 0.2), 
	     XFIG_CENTER, XFIG_TIMES_ROMAN, 24,
	     XFIG_BLACK, 0.0, s);
  }
  for (j = ys; j <= ye; j++) {
    XfigLine(fptr, XFIG(xs), YFIG((real)j), XFIG(xe), YFIG((real)j),
	     XFIG_SOLID, 1, XFIG_BLACK);
    sprintf(s, "%d", j);
    XfigText(fptr, XFIG((real)xs - 0.5), YFIG((real)j + 0.75), 
	     XFIG_CENTER, XFIG_TIMES_ROMAN, 24,
	     XFIG_BLACK, 0.0, s);
  }

  maxu = fmodmax(u, nx, ny, &i, &j);
  maxv = fmodmax(v, nx, ny, &i, &j);
  maxu = maxv > maxu ? maxv : maxu;
  maxu = maxu == 0.0 ? 1.0 : maxu;
  /*
    for (i = 2; i < nx; i++)
    for (j = 2; j < ny; j++) { */
  for (i = xs; i <= xe; i++)
    for (j = ys; j <= ye; j++) {
      if (!INU(i,j) && BU(i,j))
	XfigVector(fptr, XFIG((real)i), YFIG(0.5 + (real)j),
		   SCALE*u[i][j]/maxu, 0.0,
		   XFIG_SOLID, 1, XFIG_BLUE);
      else if (INU(i,j))
	XfigVector(fptr, XFIG((real)i), YFIG(0.5 + (real)j),
		   SCALE*u[i][j]/maxu, 0.0,
		   XFIG_SOLID, 1, XFIG_RED);
      if (!INV(i,j) && BV(i,j))
	XfigVector(fptr, XFIG(0.5 + (real)i), YFIG((real)j),
	           0.0, SCALE*v[i][j]/maxu,
	           XFIG_SOLID, 1, XFIG_GREEN);
      else if (INV(i,j))
	XfigVector(fptr, XFIG(0.5 + (real)i), YFIG((real)j),
		   0.0, SCALE*v[i][j]/maxu,
		   XFIG_SOLID, 1, XFIG_MAGENTA);
      if (INP(i,j))
	XfigBox(fptr, XFIG(0.5 + (real)i) - SCALE/20.,
		YFIG(0.5 + (real)j) - SCALE/20.,
		XFIG(0.5 + (real)i) + SCALE/20.,
		YFIG(0.5 + (real)j) + SCALE/20.,
		XFIG_SOLID, 2, XFIG_BLACK);
    }

  x = (int *)malloc(in.n*sizeof(int));
  y = (int *)malloc(in.n*sizeof(int));
  for (i = 0; i < in.n; i++) {
    x[i] = XFIG(in.x[i]);
    y[i] = YFIG(in.y[i]);
  }
  XfigPolyLine(fptr, x, y, in.n, XFIG_SOLID, 1, XFIG_BLACK);
  for (i = 0; i < in.n; i++) {
    XfigCircle(fptr, x[i], y[i], SCALE/20.,
	       XFIG_SOLID, 2, XFIG_BLACK);
    sprintf(s, "%d", i + 1);
    XfigText(fptr, x[i] + 12, y[i], 
	     XFIG_CENTER, XFIG_TIMES_ROMAN, 20,
	     XFIG_BLACK, 0.0, s);
  }
  free(x); free(y);

}







