/*----------------------------------------------------------------------------*/
/* SCCS Information: %W%    %G% */
/*----------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "utilf.h"
#include "markers.h"
#include "xfig.h"
#include "printxplot.h"
#include "inout.h"


void printxpl(real2D u, real2D v, real2D c, real2D p, 
	      int nx, int ny, 
	      real tau, real du, real dp, char *fname)
{
  int i, j, n1, n2;
  float uu, vv, cc, pp, h;
  char s[256];
  FILE *fptr = fopen(fname, "wb");
  
  h = 1.0 / (nx - 2);
  
  n1 = nx - 2;
  n2 = ny - 2;
  fwrite(&n1, sizeof(int), 1, fptr);
  fwrite(&n2, sizeof(int), 1, fptr);
  i = 1;
  fwrite(&i, sizeof(int), 1, fptr);
  i = 4;	
  fwrite(&i, sizeof(int), 1, fptr);
  strcpy(s, "u");
  fwrite((char *) s, sizeof(char), 256, fptr);
  strcpy(s, "v");
  fwrite((char *) s, sizeof(char), 256, fptr);                
  strcpy(s, "c");
  fwrite((char *) s, sizeof(char), 256, fptr);
  strcpy(s, "p");
  fwrite((char *) s, sizeof(char), 256, fptr);
  du *= 0.5 * h / tau;
  dp *= h * h / (tau * tau);

  for (i = 2; i <= nx - 1; i++)
    for (j = 2; j <= ny - 1; j++) {
      if (u[i][j] != UNDEFINED && u[i+1][j] != UNDEFINED)
	uu = (u[i][j] + u[i+1][j]) * du;
      else uu = 0.0;
      fwrite(&uu, sizeof(float), 1, fptr);
    }
  for (i = 2; i <= nx - 1; i++)
    for (j = 2; j <= ny - 1; j++) {
      if (v[i][j] != UNDEFINED && v[i][j+1] != UNDEFINED)
	vv = (v[i][j] + v[i][j+1]) * du;
      else vv = 0.0;
      fwrite(&vv, sizeof(float), 1, fptr);
    }
  for (i = 2; i <= nx - 1; i++)
    for (j = 2; j <= ny - 1; j++) {
      cc = c[i][j];
      fwrite(&cc, sizeof(float), 1, fptr);
    }
  for (i = 2; i <= nx - 1; i++)
    for (j = 2; j <= ny - 1; j++) {
      pp = p[i][j] * dp;
      fwrite(&pp, sizeof(float), 1, fptr);
    }
  
  fclose(fptr);
  
  printf("popiplot file %s written\n", fname);
}


void datbarray(real2D a, int nx, int ny, char *name)
{
  int i, j;
  FILE *fptr;
  char fname[256];
  float val;

  sprintf(fname, "%s.datb", name);
  fptr = fopen(fname, "wt");
  fwrite(&nx, sizeof(int), 1, fptr);
  fwrite(&ny, sizeof(int), 1, fptr);
  i = 1;
  fwrite(&i, sizeof(int), 1, fptr);
  fwrite(&i, sizeof(int), 1, fptr);
  strcpy(fname, "val");
  fwrite((char *) fname, sizeof(char), 256, fptr);
  for (i = 1; i <= nx; i++)
    for (j = 1; j <= ny; j++)
      {
	val = a[i][j];
	fwrite(&val, sizeof(float), 1, fptr);
      }
  fclose(fptr);
}


void datbvector(real2D u, real2D a, real2D v, real2D c, 
                int nx, int ny, char *name)
{
  int i, j;
  FILE *fptr;
  char fname[256];
  real umax = 0.0;
  
  for (i = 2; i < nx; i++)
    for (j = 2; j < ny; j++) {
      if (a[i-1][j] == 1.0 && fabs(u[i][j]) > umax)
        umax = fabs(u[i][j]);
      if (c[i][j-1] == 1.0 && fabs(v[i][j]) > umax)
        umax = fabs(v[i][j]);
    }
 
  sprintf(fname, "%s.gnu", name);
  fptr = fopen(fname, "wt");
  for (i = 2; i < nx; i++)
    for (j = 2; j < ny; j++)
      if (umax > 0.0) {
        if (a[i-1][j] == 1.0)
	  fprintf(fptr, "%g %g\n%g %g\n\n", 
	  	  (real)i, 0.5 + (real)j, 
		  u[i][j]/umax + (real)i, 0.5 + (real)j);
	if (c[i][j-1] == 1.0)
	  fprintf(fptr, "%g %g\n%g %g\n\n", 
		  (real)i + 0.5, (real)j, 
		  (real)i + 0.5, (real)j + v[i][j]/umax);
      }
      else {
	fprintf(fptr, "%g %g\n%g %g\n\n", 
		(real)i, 0.5 + (real)j, 
		(real)i, 0.5 + (real)j);
	fprintf(fptr, "%g %g\n%g %g\n\n", 
		(real)i + 0.5, (real)j, 
		(real)i + 0.5, (real)j);
      }
  fclose(fptr);
}


void gnuvectors(real2D u, real2D a, real2D v, real2D c, 
                int nx, int ny, char *name)
{
  int i, j;
  FILE *fptr;
  char fname[256];
  real umax = 0.0, u1, v1;
  
  for (i = 2; i < nx; i++)
    for (j = 2; j < ny; j++) {
      if (a[i-1][j] == 1.0 && fabs(u[i][j]) > umax)
        umax = fabs(u[i][j]);
      if (c[i][j-1] == 1.0 && fabs(v[i][j]) > umax)
        umax = fabs(v[i][j]);
    }
 
  sprintf(fname, "%s.gnu", name);
  fptr = fopen(fname, "wt");
  for (i = 2; i < nx; i++)
    for (j = 2; j < ny; j++)
      if (umax > 0.0) {
        if (a[i][j] <= 1.0) {
          if (a[i-1][j] <= 1.0)
            u1 = 0.5*(u[i][j] + u[i+1][j]);
          else
            u1 = u[i+1][j]; 
        }
        else if (a[i-1][j] == 1.0)
          u1 = u[i][j];
        else
          u1 = 1000.0;
        if (c[i][j] <= 1.0) {
          if (c[i][j-1] <= 1.0)
            v1 = 0.5*(v[i][j] + v[i][j+1]);
          else
            v1 = v[i][j+1];
        }
        else if (c[i][j-1] == 1.0)
          v1 = v[i][j];
        else
          v1 = 1000.0;
        if (u1 != 1000.0 && v1 != 1000.0)
	  fprintf(fptr, "%g %g\n%g %g\n\n", 
	  	  0.5 + (real)i, 0.5 + (real)j, 
		  3.*u1/umax + 0.5 + (real)i, 3.*v1/umax + 0.5 + (real)j);
      }
      else {
	fprintf(fptr, "%g %g\n%g %g\n\n", 
		0.5 + (real)i, 0.5 + (real)j, 
		0.5 + (real)i, 0.5 + (real)j);
	fprintf(fptr, "%g %g\n%g %g\n\n", 
		(real)i + 0.5, 0.5 + (real)j, 
		(real)i + 0.5, 0.5 + (real)j);
      }
  fclose(fptr);
}






