/*------------------------------------------------------------------*/
/* SCCS Information: %W%    %G% */
/*------------------------------------------------------------------*/
#include "utilf.h"
#include "markers.h"
#include "surfaces.h"
#include "extra.h"
#include "global.h"
#include <math.h>
#include <stdio.h>
#include <readSIunits.h>

void readglob()
{
  real lambdasi, rholiqsi,
    visliqsi, timescalesi, xvelsi, yvelsi,
    sigmasi, gsi, xgsi, ygsi;
  real posi, pesi, p_infsi, omegasi;
  real diameter, Rosi;

  /*-----------------------------------------------------------------*/
  /*                   Numerical parameters                          */
  /*-----------------------------------------------------------------*/
  
  if (!intread("nx", &num.nx, stdin))
    {  printf("missing nx\n"); exit(1);  }
  
  if (!intread("ny", &num.ny, stdin))
    {  printf("missing ny\n"); exit(1);  }
  
  if (!intread("tmax", &num.tmax, stdin))
    {  printf("missing tmax\n"); exit(1);  }
  
  if (!realreadSI("tau-si", &num.tausi, stdin)) 
    {  printf("missing tau-si\n"); exit(1);  }

  /* Markers redistribution length */
  if (!realreadSI("lredis", &num.lredis, stdin))
    {  printf("missing lredis\n"); exit(1); }
  
  if (!intread("cycles", &num.cycles, stdin))
    {  printf("missing cycles\n"); exit(1);  }
  
  if (!intread("npre", &num.npre, stdin))
    {  printf("missing npre\n"); exit(1);  }
  
  if (!intread("npost", &num.npost, stdin))
    {  printf("missing npost\n"); exit(1);  }

  if (!realreadSI("maxdiv", &num.maxdiv, stdin)) 
    {  printf("missing maxdiv\n"); exit(1);  }

  /*-----------------------------------------------------------------*/
  /*                   Physical parameters                           */ 
  /*-----------------------------------------------------------------*/
  
  if (!realreadSI("L-si", &lambdasi, stdin))
    {  printf("missing L-si\n"); exit(1);  }
  
  if (!realreadSI("rholiq-si", &rholiqsi, stdin)) 
    {  printf("missing rholiq-si\n"); exit(1);  }
  
  if (!realreadSI("visliq-si", &visliqsi, stdin))  
    {  printf("missing visliq-si\n"); exit(1);  }
  
  if (!realreadSI("sigma-si", &sigmasi, stdin))  
    {  printf("missing sigma-si\n"); exit(1);  }

  if (!realreadSI("po-si", &posi, stdin))
    {  printf("missing po-si\n"); exit(1);  }

  if (!realreadSI("Ro-si", &Rosi, stdin))
    {  printf("missing Ro-si\n"); exit(1);  }

  if (!intread("eq", &phys.eq, stdin))
    {  printf("missing eq\n"); exit(1);  }

  if (!phys.eq && !realreadSI("p_inf-si", &p_infsi, stdin))
    {  printf("missing p_inf-si\n"); exit(1);  }

  if (!realreadSI("pe-si", &pesi, stdin))
    {  printf("missing pe-si\n"); exit(1);  }

  if (!realreadSI("omega-si", &omegasi, stdin))
    {  printf("missing omega-si\n"); exit(1);  }

  if (!realreadSI("gamma", &phys.gamma, stdin))
    {  printf("missing gamma\n"); exit(1);  }

  xgsi = ygsi = 0.0;
  realreadSI("gx-si", &xgsi, stdin);
  realreadSI("gy-si", &ygsi, stdin);      
  gsi = sqrt(xgsi * xgsi + ygsi * ygsi);
  
  xvelsi = yvelsi = 0.0;
  realreadSI("xvel-si", &xvelsi, stdin);  
  realreadSI("yvel-si", &yvelsi, stdin);
  
  /*-----------------------------------------------------------------*/
  /*                   Dimensionless units                           */ 
  /*-----------------------------------------------------------------*/
  
  printf("############ SI Units ############\n");
  if(gsi != 0.0)
    {
      timescalesi = sqrt(lambdasi / fabs(gsi));
      phys.gx = xgsi / gsi;
      phys.gy = ygsi / gsi;
    }
  else 
    {
      timescalesi = 1.0;
      phys.gx = phys.gy = 0.0;
    }
  printf("timescale-si=%g\n", timescalesi);
  
  num.tau = num.tausi / timescalesi;
  phys.t = 0.0;
  phys.visliq = visliqsi * timescalesi / (rholiqsi * sq(lambdasi));
  phys.sigma = sigmasi * sq(timescalesi) /
    (rholiqsi * lambdasi * lambdasi * lambdasi);
  phys.po = posi*sq(timescalesi)/(rholiqsi*sq(lambdasi));
  Rosi = Rosi/lambdasi*((real)num.nx - 2.);
  phys.vo = 4.*PI*cube(Rosi)/3.;
  phys.pe = pesi*sq(timescalesi)/(rholiqsi*sq(lambdasi));
  phys.p_inf = p_infsi*sq(timescalesi)/(rholiqsi*sq(lambdasi));
  phys.omega = omegasi*timescalesi;
  phys.xvel = xvelsi * timescalesi / lambdasi;
  phys.yvel = yvelsi * timescalesi / lambdasi;
  num.du = lambdasi / timescalesi;
  num.dp = (rholiqsi * sq(lambdasi)) / sq(timescalesi);

  if (phys.eq) {
    if (!realreadSI("diameter", &diameter, stdin)) 
      {  printf("missing diameter\n"); exit(1);  }
    phys.pext = phys.po - 4.*phys.sigma/diameter;
  }
  else
    phys.pext = phys.p_inf;
  
  printf("lambda-si=%g\n", lambdasi);
  printf("rholiq-si=%g\n", rholiqsi);
  printf("visliq-si=%g\n", visliqsi);
  printf("sigma-si=%g\n", sigmasi);
  printf("xvel-si=%g\n", xvelsi);
  printf("yvel-si=%g\n", yvelsi);
  printf("gx-si=%g\n", xgsi);
  printf("gy-si=%g\n", ygsi);
  printf("tau-si=%g\n", num.tausi);
  printf("dp=%g\n", num.dp);
  
  /*-----------------------------------------------------------------*/
  /*                   Output parameters                             */ 
  /*-----------------------------------------------------------------*/
  
  if (!intread("tprint", &out.tprint, stdin))
      {  printf("missing tprint\n"); exit(1);  }
  if (!intread("tsave", &out.tsave, stdin))
    {  printf("missing tsave\n"); exit(1);  }
  if (!intread("txplot", &out.txplot, stdin))
    {  printf("missing txplot\n"); exit(1);  }
  if (!intread("tmark", &out.tmark, stdin))
    {  printf("missing tmark\n"); exit(1);  }
}


void printhist()
{
  printf("############ Dimensionless Units ############\n");
  printf("----------- Physical parameters -------------\n");
  printf("rholiq=%g\n", 1.0);
  printf("visliq=%g\n", phys.visliq);
  printf("sigma=%g\n", phys.sigma);
  printf("p_inf=%g\n", phys.p_inf);
  printf("pe=%g\n", phys.pe);
  printf("po=%g\n", phys.po);
  printf("vo=%g\n", phys.vo);
  printf("gamma=%g\n", phys.gamma);
  printf("xvel=%g\n", phys.xvel);
  printf("yvel=%g\n", phys.yvel);
  printf("gx=%g\n", phys.gx);
  printf("gy=%g\n", phys.gy);
  printf("----------- Numerical parameters ------------\n");
  printf("nx x ny=%d x %d\n", num.nx, num.ny);    
  printf("tau=%g\n", num.tau);
  printf("cycles=%d npre=%d npost=%d maxdiv=%g\n", 
	 num.cycles, num.npre, num.npost, num.maxdiv);
} 


void printnumbers()
{
  real vel;
  printf("############# Dimensionless Numbers ##############\n");
  printf("Ohnesorge's Number: 1/Oh^2=sigma*rho*L/mu^2\n");
  printf("  %g\n", phys.sigma/(phys.visliq*phys.visliq));
  vel = sqrt(sq(phys.xvel) + sq(phys.yvel));
  printf("Reynolds Number: Re=rho*v*L/mu\n");
  printf("  %g\n", vel/phys.visliq);
  if (phys.sigma > 0.0) {
    printf("Capilary Number: Ca=mu*v/sigma\n");
    printf("  %g\n", phys.visliq*vel/phys.sigma);
    printf("Weber Number: We=rho*v^2*d/sigma\n");
    printf("  %g\n", sq(vel)/phys.sigma);
  }
  printf("------------- Numerical numbers -----------------\n");
  if (phys.sigma > 0.0) {
    real dx = 1.0 / (num.nx - 2);
    printf("Capillary Wave Number: Cw=tau/sqrt(rho*dx^3/sigma)\n");
    printf("  %g\n", num.tau/sqrt(dx*dx*dx/phys.sigma));
  }
}

      











