/*----------------------------------------------------------------------------*/
/* SCCS Information: %W%    %G% */
/*----------------------------------------------------------------------------*/
#include <math.h>
#include <stdio.h>
#include "utilf.h"
#include "inout.h"


real findmax(real2D u, int nx, int ny, int *imax, int *jmax)
{
  int i, j;
  real res;
  real fmax;
  
  fmax = -1.0;
  for (i = 2; i <= nx - 1; i++)
    for (j = 2; j <= ny - 1; j++)
      {
	res = u[i][j];
	if (res > fmax)
	  {
	    fmax = res;
	    *imax = i;
	    *jmax = j;
	  }
      }
  
  return fmax;
}


real fmean(real2D u, real umean, int xs, int nx, int ny)
{
  int i, j;
  real mean = 0;
  
  for (i = xs; i <= nx - 1; i++)
    for (j = 2; j <= ny - 1; j++)
      mean += (u[i][j] - umean) * (u[i][j] - umean);

  return sqrt(mean / (nx - xs) / (ny - 2));
}


real fmodmax(real2D u, int nx, int ny, int *imax, int *jmax)
{
  int i, j;
  real res, fmax;
  
  fmax = 0.0;
  for (i = 2; i <= nx - 1; i++)
    for (j = 2; j <= ny - 1; j++)
      if (u[i][j] != UNDEFINED) {
	res = u[i][j];
	if(fabs(res) > fmax) {
	  fmax = fabs(res);
	  *imax = i;
	  *jmax = j;
	}
      }
  
  return fmax;
}


real minau(real2D au, real2D ap, int nx, int ny, int *imin, int *jmin)
{
  int i, j;
  real val = 100.;
  for (i = 2; i < nx; i++)
    for (j = 2; j < ny; j++)
      if (INU(i,j))
	if (au[i][j] < val) {
	  val = au[i][j];
          *imin = i; *jmin = j;
	}
  return val;
}


real minav(real2D av, real2D ap, int nx, int ny, int *imin, int *jmin)
{
  int i, j;
  real val = 100.;
  for (i = 2; i < nx; i++)
    for (j = 3; j < ny; j++)
      if (INV(i,j))
	if (av[i][j] < val) {
	  val = av[i][j];
          *imin = i; *jmin = j;
	}
  return val;
}


real findmin(real2D u, int nx, int ny, int *imin, int *jmin)
{
  int i, j;
  real res, fmin;
  
  fmin = 1.0;
  for (i = 2; i <= nx - 1; i++)
    for (j = 2; j <= ny - 1; j++)
      {
	res = u[i][j];
	if(res < fmin)
	  {
	    fmin = res;
	    *imin = i;
	    *jmin = j;
	  }
      }
      
  return fmin;
}


real sumfield(real2D u, int nx, int ny)
{
  int i, j;
  real sum;
  
  sum = 0.0;
  for (i = 2; i <= nx - 1; i++)
    for (j = 2; j <= ny - 1; j++)
      sum += u[i][j]; 
  
  return sum;
}


real dp(real2D p, real2D cc, int nx, int ny)
{
  int i, j, nin = 0, nout = 0;
  real pin = 0, pout = 0;

  for (i = 2; i <= nx - 1; i++)
    for (j = 2; j <= ny - 1; j++)
      if (cc[i][j] == 1.0) {
	pin += p[i][j]; nin++;
      } 
      else 
	if (cc[i][j] == 0.0) {
	  pout += p[i][j]; nout++;
	}
  if (nin > 0 && nout > 0)
    return pin / (real) nin - pout / (real) nout;
  else
    return 0.0;
}


/* compute the maximum relative error on divergence i.e.
   max(abs(div()/volume())) */
real rdivmax(real2D div, int nx, int ny, int *imax, int *jmax)
{
  int i, j;
  real val, maxdiv = 0.0;

  for (i = 2; i < nx; i++)
    for (j = 2; j < ny; j++) {
      /* 2D-axi: the divergence is div, the volume is (r + 1/2) */
      val = fabs(div[i][j]/((real)j - 1.5));
      if (val > maxdiv) {
	maxdiv = val; *imax = i; *jmax = j;
      }
    }
  return maxdiv;
}


void copy(real2D out, real2D in, int nx, int ny)
{
  int i, j;
  for (i = 1; i <= nx; i++)
    for (j = 1; j <= ny; j++)
      out[i][j] = in[i][j];
}


void add(real2D a, real2D b, int nx, int ny)
{
  int i, j;
  
  for (i = 1; i <= nx; i++)
    for (j = 1; j <= ny; j++)
      a[i][j] += b[i][j];
}


void fill0(real2D u, int nx, int ny)
{      
  int i, j;
  for (i = 1; i <= nx; i++)
    for (j = 1; j <= ny; j++)
      u[i][j] = 0.0;
}



