#include <stdio.h>
#include <math.h>
#include "xfig.h"

#define ARROWRATIO 3.0
#define ARROWHW 0.5

void XfigStartCompound(FILE *fptr, int ix1, int iy1, int ix2, int iy2)
{
  fprintf(fptr, "%d %d %d %d %d\n", XFIG_COMPOUND,
	  ix1, iy1, ix2, iy2);
}


void XfigEndCompound(FILE *fptr)
{
  fprintf(fptr, "%d\n", - XFIG_COMPOUND);
}


void XfigPolyLine(FILE *fptr, int *x, int *y, int npoints, 
		  int linestyle, int thickness, int color)
{
  int i;
  fprintf(fptr, "%d %d %d %d %d 0 -1 %d %f 0 0 0\n",
	  XFIG_POLYLINE, XFIG_POLY_TYPE, linestyle, thickness, color,
	  XFIG_NOT_FILLED, 0.0);
  for (i = 0; i < npoints; i++)
    fprintf(fptr, "%d %d\n", x[i], y[i]);
  fprintf(fptr, "%d %d\n", XFIG_END_POLY, XFIG_END_POLY);
}


void XfigVector(FILE *fptr, int x, int y, int fx, int fy, 
		int linestyle, int thickness, int color)
{
  double fl, arrowheight;

  fprintf(fptr, "%d %d %d %d %d 0 -1 %d %f 0 1 0\n",
	  XFIG_POLYLINE, XFIG_POLY_TYPE, linestyle, thickness, color,
	  XFIG_NOT_FILLED, 0.0);
  fl = sqrt(fx*fx + fy*fy);
  arrowheight = fl/ARROWRATIO;
  fprintf(fptr, "-1 -1 %.3f %.3f %.3f\n", 1.0, 
	  ARROWHW*arrowheight, arrowheight);
  fprintf(fptr, "%d %d %d %d %d %d\n", x, y, x + fx, y + fy,
	  XFIG_END_POLY, XFIG_END_POLY);
}


void XfigLine(FILE *fptr, int x, int y, int x1, int y1, 
	      int linestyle, int thickness, int color)
{
  fprintf(fptr, "%d %d %d %d %d 0 -1 %d %f 0 0 0\n",
	  XFIG_POLYLINE, XFIG_POLY_TYPE, linestyle, thickness, color,
	  XFIG_NOT_FILLED, 0.0);
  fprintf(fptr, "%d %d %d %d %d %d\n", x, y, x1, y1,
	  XFIG_END_POLY, XFIG_END_POLY);
}


void XfigBox(FILE *fptr, int ix1, int iy1, int ix2, int iy2,
	     int linestyle, int thickness, int color)
{
  fprintf(fptr, "%d %d %d %d %d 0 -1 %d %f 0 0 0\n",
	  XFIG_BOX_TYPE, XFIG_BOX_TYPE,
	  linestyle, thickness, color, XFIG_NOT_FILLED, 0.0);
  fprintf(fptr, "%d %d %d %d %d %d %d %d %d %d %d %d\n",
	  ix1, iy1, ix1, iy2, ix2, iy2, ix2, iy1, ix1, iy1, 
	  XFIG_END_POLY, XFIG_END_POLY);
}


void XfigCircle(FILE *fptr, int x, int y, int radius,
		int linestyle, int thickness, int color)
{
  fprintf(fptr, "%d 3 %d %d %d 0 -1 %d %f 1 0.0 %d %d %d %d %d %d %d %d\n",
	  XFIG_ELLIPSE_TYPE, linestyle, thickness, color, XFIG_BLACK_FILLED,
	  0.0,
	  x, y, radius, radius, x, y, x + radius, y);
}


void XfigText(FILE *fptr, int x, int y, int sub_type, int font, int font_size,
	      int color, float angle, char *text)
{
  fprintf(fptr, "%d %d %d %d -1 %d 0 %f %d %d %d %d %d %s\1\n",
	  XFIG_TEXT_TYPE, sub_type, font, font_size, color, angle, 
	  4, font_size, strlen(text)*font_size, x, y, text);
}

